
#define	NEKO

#include	<windows.h>
#include	"fddfile.h"
#include	"fdd_d88.h"

#ifdef KEROPI
#include	"../win32/common.h"
#include	"../win32/fileio.h"
#include	"fdc.h"

#define		FILEHDL		FILEH
#define		FILE_OPEN	File_Open
#define		FILE_READ	File_Read
#define		FILE_SEEK	File_Seek
#define		FILE_CLOSE	File_Close
#endif

#ifdef NEKO
#include	"common.h"
#include	"np2.h"
#include	"dosio.h"
#include	"iobridge.h"
#include	"fdc.h"

#define		FILEHDL		FILEH
#define		FILE_OPEN	file_open
#define		FILE_READ	file_lread
#define		FILE_WRITE	file_lwrite
#define		FILE_SEEK	file_seek
#define		FILE_CLOSE	file_close
#endif

#define		D88BUFSIZE	0x4000

static	D88_HEADER	d88head[4];
static	D88_SECTOR	cursec;
static	BYTE		curdrv = 0xff;
static	WORD		curtrk = 0xffff;
static	BYTE		curwrite = 0;
static	DWORD		curfp = 0;
static	DWORD		cursize = 0;
static	BYTE		curtrkaccess = 0;
static	BYTE		curtype = 0;

static	BYTE		D88_BUF[D88BUFSIZE];

// ------------------------------------------------------------------------

static BOOL curdataflush(void) {

	FILEHDL	hdl;
	BOOL	ret = SUCCESS;

	if ((!curfp) || (!cursize) || (!curwrite)) {
		return(SUCCESS);
	}
	curwrite = 0;
	if ((hdl = FILE_OPEN(fddfile[curdrv].fname)) == (FILEHDL)-1) {
		return(FAILURE);
	}
	if ((FILE_SEEK(hdl, curfp, FSEEK_SET) != curfp) ||
		(FILE_WRITE(hdl, D88_BUF, cursize) != cursize)) {
		ret = FAILURE;
	}
	if (FILE_CLOSE(hdl)) {
		ret = FAILURE;
	}
	return(ret);
}


static void drvflush(BYTE drv) {

	if (curdrv == drv) {
		curdataflush();
		curdrv = 0xff;
		curtrk = 0xffff;
	}
}

static DWORD nexttrackp(D88_HEADER *head, DWORD fptr, DWORD last) {

	int		t;
	DWORD	ret;
	DWORD	*trkp;

	ret = last;
	trkp = (DWORD *)head->trackp;
	for (t=0; t<164; t++, trkp++) {
		if ((*trkp > fptr) && (*trkp < ret)) {
			ret = *trkp;
		}
	}
	return(ret);
}

static BOOL read_d88track(BYTE drv, BYTE track, BYTE type) {

	FILEHDL	hdl;

	curdataflush();
	curdrv = drv;
	curtrk = track;
	curtype = type;
	curwrite = 0;
	curtrkaccess = SUCCESS;

	if ((drv >= 4) || (track > 163) ||
		(type != (d88head[drv].fd_type >> 4)) ||
		((curfp = d88head[drv].trackp[track]) == 0) ||
		((cursize = nexttrackp(&d88head[drv], curfp,
							d88head[drv].fd_size) - curfp) > D88BUFSIZE)) {
		goto readd88_err;
	}
	if ((hdl = FILE_OPEN(fddfile[drv].fname)) == (FILEH)-1) {
		goto readd88_err;
	}
	if (FILE_SEEK(hdl, curfp, FSEEK_SET) != curfp) {
		FILE_CLOSE(hdl);
		goto readd88_err;
	}
	if (FILE_READ(hdl, D88_BUF, cursize) != cursize) {
		FILE_CLOSE(hdl);
		goto readd88_err;
	}
	if (FILE_CLOSE(hdl)) {
		goto readd88_err;
	}
	return(SUCCESS);

readd88_err:
	curfp = 0;
	cursize = 0;
	curtrkaccess = FAILURE;
	return(FAILURE);
}

static BOOL seek_d88(BYTE drv, BYTE track) {

	if ((curdrv != drv) || (curtrk != track) || (curtype != CTRL_FDMEDIA)) {
		read_d88track(drv, track, CTRL_FDMEDIA);
	}
	return(curtrkaccess);
}

static D88_SECTOR *searchsector_d88(void) {

	BYTE	*p;
	BYTE	sec;
	WORD	pos = 0;

	p = D88_BUF;
	for (sec=0; sec<40; ) {
		if ((((D88_SECTOR *)p)->c == fdc.C) &&
			(((D88_SECTOR *)p)->h == fdc.H) &&
			(((D88_SECTOR *)p)->r == fdc.R) &&
			(((D88_SECTOR *)p)->n == fdc.N)) {
			return((D88_SECTOR *)p);
		}
		if (++sec >= ((D88_SECTOR *)p)->sectors) {
			break;
		}

		pos += ((D88_SECTOR *)p)->size;
		pos += sizeof(D88_SECTOR);

		p += ((D88_SECTOR *)p)->size;
		p += sizeof(D88_SECTOR);
	}
	return(NULL);
}

// ------------------------------------------------------------------------

BOOL fdd_set_d88(BYTE drv, LPSTR fname) {

	WORD	attr;
	FILEHDL	hdl;
	DWORD	rsize;

	fdd_eject_d88(drv);
	if ((attr = file_attr(fname)) & 0x18) {
		return(FAILURE);
	}
	if ((hdl = FILE_OPEN(fname)) == (FILEH)-1) {
		return(FAILURE);
	}
	rsize = FILE_READ(hdl, &d88head[drv], sizeof(D88_HEADER));
	FILE_CLOSE(hdl);
	if (rsize != sizeof(D88_HEADER)) {
		fdd_eject_d88(drv);
		return(FAILURE);
	}
	strcpy(fddfile[drv].fname, fname);
	fddfile[drv].type = DISKTYPE_D88;
	fddfile[drv].protect = FALSE;
	if ((attr & 1) || (d88head[drv].protect & 0x10)) {
		fddfile[drv].protect = TRUE;
	}
	return(SUCCESS);
}

BOOL fdd_eject_d88(BYTE drv) {

	drvflush(drv);
	ZeroMemory(&d88head[drv], sizeof(D88_HEADER));
	fddfile[drv].fname[0] = '\0';
	fddfile[drv].type = DISKTYPE_NOTREADY;
	return(SUCCESS);
}


BOOL fdd_diskaccess_d88(void) {

	if (CTRL_FDMEDIA != (d88head[fdc.us].fd_type >> 4)) {
		return(FAILURE);
	}
	return(SUCCESS);
}

BOOL fdd_seek_d88(void) {

	if (fdc.ncn >= 82) {
		return(FAILURE);
	}
	return(seek_d88(fdc.us, (fdc.ncn << 1) + fdc.hd));
}

BOOL fdd_seeksector_d88(void) {

	if (fdc.treg[fdc.us] >= 0x80) {
		return(FAILURE);
	}
	if (seek_d88(fdc.us, (fdc.treg[fdc.us] << 1) + fdc.hd)) {
		return(FAILURE);
	}
	if (!searchsector_d88()) {
		return(FAILURE);
	}
	return(SUCCESS);
}

BOOL fdd_read_d88(void) {

	D88_SECTOR	*p;
	WORD		size;

	if (fdc.treg[fdc.us] >= 0x80) {
		return(FAILURE);
	}
	if (seek_d88(fdc.us, (fdc.treg[fdc.us] << 1) + fdc.hd)) {
		return(FAILURE);
	}
	p = searchsector_d88();
	if (!p) {
		return(FAILURE);
	}
	if (fdc.N < 8) {
		size = 128 << fdc.N;
	}
	else {
		size = 128 << 8;
	}
	fdc.bufcnt = size;
	ZeroMemory(fdc.buf, size);
	if (size > p->size) {
		size = p->size;
	}
	if (size) {
		memcpy(fdc.buf, p+1, size);
	}
	return(SUCCESS);
}

BOOL fdd_write_d88(void) {

	D88_SECTOR	*p;
	WORD		size;

	if (fdc.treg[fdc.us] >= 0x80) {
		return(FAILURE);
	}
	if (seek_d88(fdc.us, (fdc.treg[fdc.us] << 1) + fdc.hd)) {
		return(FAILURE);
	}
	p = searchsector_d88();
	if (!p) {
		return(FAILURE);
	}
	if (fdc.N < 8) {
		size = 128 << fdc.N;
	}
	else {
		size = 128 << 8;
	}
	if (size > p->size) {
		size = p->size;
	}
	if (size) {
		memcpy(p+1, fdc.buf, size);
		curwrite = TRUE;
	}
	return(SUCCESS);
}

BOOL fdd_readid_d88(void) {

	BYTE	*p;
	BYTE	sec;

	if (fdc.treg[fdc.us] >= 0x80) {
		return(FAILURE);
	}
	if (seek_d88(fdc.us, (fdc.treg[fdc.us] << 1) + fdc.hd)) {
		return(FAILURE);
	}

	p = D88_BUF;
	for (sec=0; sec<40; ) {
		if (sec == fdc.crcn) {
			fdc.C = ((D88_SECTOR *)p)->c;
			fdc.H = ((D88_SECTOR *)p)->h;
			fdc.R = ((D88_SECTOR *)p)->r;
			fdc.N = ((D88_SECTOR *)p)->n;
			fdc.crcn++;
			return(SUCCESS);
		}
		if (++sec >= ((D88_SECTOR *)p)->sectors) {
			break;
		}
		p += ((D88_SECTOR *)p)->size;
		p += sizeof(D88_SECTOR);
	}
	fdc.crcn = 0xff;
	return(FAILURE);
}

