#include	<windows.h>
#include	<windowsx.h>

#include	"common.h"
#include	"np2.h"
#include	"dosio.h"
#include	"menu.h"
#include	"s98.h"
#include	"nevent.h"
#include	"pc_core.h"

#define S98LOG_BUFSIZE (32*1024)
static struct {
	FILEH fh;
	DWORD intcount;
	DWORD clock;
	DWORD p;
	BYTE buf[S98LOG_BUFSIZE];
} s98log;

void s98timer(NEVENT_ITEM *item);

static void sets98event(BOOL absolute) {
	s98log.intcount++;
	nevent_set(NEVENT_S98TIMER, s98log.clock, s98timer, NEVENT_RELATIVE);
}

void s98timer(NEVENT_ITEM *item) {
	if (s98log.fh == (FILEH)-1) return;
	sets98event(NEVENT_RELATIVE);
}

static void S98_frush(void)
{
	file_lwrite(s98log.fh, s98log.buf, s98log.p);
	s98log.p = 0;
}

static void S98_putc(BYTE data)
{
	s98log.buf[s98log.p++] = data;
	if (s98log.p == S98LOG_BUFSIZE) S98_frush();
}
static void S98_putint(void)
{
	if (s98log.intcount)
	{
		if (s98log.intcount == 1)
		{
			S98_putc(0xFF);	/* SYNC(1) */
		}
		else if (s98log.intcount == 2)
		{
			S98_putc(0xFF);	/* SYNC(1) */
			S98_putc(0xFF);	/* SYNC(1) */
		}
		else
		{
			S98_putc(0xFE);	/* SYNC(n) */
			s98log.intcount -= 2;
			while (s98log.intcount > 0x7f)
			{
				S98_putc((BYTE)(0x80 | (s98log.intcount & 0x7f)));
				s98log.intcount >>= 7;
			}
			S98_putc((BYTE)(s98log.intcount & 0x7f));
		}
		s98log.intcount  = 0;
	}
}

void S98_put(BYTE ms, BYTE adr, BYTE data)
{
	if (s98log.fh != (FILEH)-1)
	{
		S98_putint();
		S98_putc(ms);
		S98_putc(adr);
		S98_putc(data);
	}
}

static int S98_open(char *filename)
{
	int i;
	if ((s98log.fh = file_create(filename)) == (FILEH)-1) return 0;
	s98log.clock = pc.realclock / 1000;
	s98log.p = 0;
	/* HEADER */
	S98_putc('S');
	S98_putc('9');
	S98_putc('8');
	S98_putc('1');
	S98_putc(0x01);	/* SYNC TIME */
	S98_putc(0x00);
	S98_putc(0x00);
	S98_putc(0x00);
	for (i = 0; i < 8; i++) S98_putc(0x00);
	S98_putc(0x40);	/* TITLE OFFSET */
	S98_putc(0x00);
	S98_putc(0x00);
	S98_putc(0x00);
	S98_putc(0x80);	/* DATA OFFSET */
	S98_putc(0x00);
	S98_putc(0x00);
	S98_putc(0x00);
	for (i = 0; i < 0x80-24; i++) S98_putc(0x00);
	sets98event(NEVENT_ABSOLUTE);
	return 1;
}

static void S98_close(void)
{
	if (s98log.fh != (FILEH)-1)
	{
		S98_putint();
		S98_putc(0xFD);	/* END MARK */
		S98_frush();
		nevent_reset(NEVENT_S98TIMER);
		file_close(s98log.fh);
		s98log.fh = (FILEH)-1;
	}
}

int S98_logging(void)
{
	LPSTR r;
	if (s98log.fh != (FILEH)-1)
	{
		S98_close();
		return 0;
	}
	if ((r = savefileselect("S98O\0*.s98\0\0", "NP2_%04d.S98", "s98", "S98̕ۑ")) != NULL) {
		return S98_open(r);
	}
	return 0;
}



void S98init(void)
{
	s98log.fh = (FILEH)-1;
}
void S98trash(void)
{
	S98_close();
}
